@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2014-2021 Christian Schoenebeck
+ * Copyright (c) 2014-2025 Christian Schoenebeck
  *
  * http://www.linuxsampler.org
  *
@@ -85,7 +85,7 @@ namespace LinuxSampler {
        STMT_ABORT_SIGNALLED = 1, ///< VM should stop the current callback execution (usually because of an error, but might also be without an error reason, i.e. when the built-in script function exit() was called).
        STMT_SUSPEND_SIGNALLED = (1<<1), ///< VM supended execution, either because the script called the built-in wait() script function or because the script consumed too much execution time and was forced by the VM to be suspended for some time
        STMT_ERROR_OCCURRED = (1<<2), ///< VM stopped execution due to some script runtime error that occurred
-       STMT_RETURN_SIGNALLED = (1<<3), ///< VM should unwind stack and return to previous, calling subroutine (i.e. user function or event handler).
+       STMT_RETURN_SIGNALLED = (1<<3), ///< VM should unwind op stack and data stack and return to previous, calling subroutine (i.e. user function or event handler).
     };
 
     /** @brief Virtual machine execution status.
@@ -172,6 +172,22 @@ namespace LinuxSampler {
         VM_ALWAYS,
     };
 
+    /**
+     * Considered use case environments for running an instance of a virtual
+     * machine.
+     *
+     * This provides an optional hint to the virtual machine of the purpose it's
+     * currently deployed for. It may use this information to adapt its
+     * behaviour accordingly.
+     *
+     * @see ScriptVM::setDeploymentCase()
+     */
+    enum VMDeploymentCase {
+        VM_AUTOMATED_TESTS,  ///< VM is running as part of fully automated test cases.
+        VM_LIVE_TESTING,     ///< VM is currently being tested manually by a person.
+        VM_PRODUCTION,       ///< VM is currently running in a production environment.
+    };
+
     // just symbol prototyping
     class VMIntExpr;
     class VMRealExpr;
@@ -376,6 +392,15 @@ namespace LinuxSampler {
          */
         VMStringExpr* asString() const;
 
+        /**
+         * Calling this method evaluates the expression and returns its result
+         * value automatically casted to a string presentation.
+         *
+         * Unlike the as*() methods, it is therefore always safe to call this
+         * method no matter what the actual expression type is.
+         */
+        virtual String evalCastToStr();
+
         /**
          * In case this expression is an integer array expression, then this
          * method returns a casted pointer to that VMIntArrayExpr object. It
@@ -1874,6 +1899,24 @@ namespace LinuxSampler {
         return "invalid";
     }
 
+    /**
+     * Convenience function used for converting a MetricPrefix_t constant to a
+     * string.
+     */
+    inline String metricPrefixStr(const MetricPrefix_t& prefix) {
+        switch (prefix) {
+            case VM_NO_PREFIX: return "none";
+            case VM_KILO: return "kilo";
+            case VM_HECTO: return "hecto";
+            case VM_DECA: return "deca";
+            case VM_DECI: return "deci";
+            case VM_CENTI: return "centi";
+            case VM_MILLI: return "milli";
+            case VM_MICRO: return "micro";
+        }
+        return "invalid";
+    }
+
     /** @brief Virtual machine representation of a script.
      *
      * An instance of this abstract base class represents a parsed script,
